from easyshell import *
from easyshell import __doc__ as doc1
from pyg import Mail, mail, AddEvent, addevent, Sheet
from pyg import __doc__ as doc2
from version import __version__

__doc__ = doc1 + '\n\n\n\n' + doc2
